package practicas;

/**
 * Clase: Enemigo
 * Descripcin: Esta clase representa un enemigo en el juego del comecocos
 * Creada por: Daniel Prez
 * Fecha: 10-may-2007
 */

public class Enemigo extends Individuo {
    public static final int COLOR_ROJO = 1;
    public static final int COLOR_NARANJA = 2;
    public static final int COLOR_AMARILLO = 3;
    public static final int COLOR_VERDE = 4;
    public static final int COLOR_CELESTE = 5;
    public static final int COLOR_AZUL = 6;
    public static final int COLOR_VIOLETA = 7;
    public static final int COLOR_ROSA = 8;
	
    /** Variable para el identificador nico del enemigo */
    private int idEnemigo;

    /** Variable para el nombre del enemigo */
    private String nombre;
    
    /** Variable para el color del enemigo */
    private int color;

    /** Variable para la distancia del enemigo al protagonista*/
    private int distancia;

    /** Contador esttico para llevar la cuenta de cuntos enemigos se han creado hasta ahora */
    private static int contadorEnemigos = 0;

    /**
     * Constructor de un determinado enemigo. Cada enemigo tiene un identificador nico, una
     * posicin y una direccin inicial
     * @param posicionX Posicin inicial X del enemigo
     * @param posicionY Posicin inicial Y del enemigo
     * @param direccion Direccin inicial del enemigo
     */
    public Enemigo(int posicionX, int posicionY, int direccion) {
        super(posicionX, posicionY, direccion);
        contadorEnemigos++;
        this.idEnemigo = contadorEnemigos;
    }

    /**
     * Constructor de un determinado enemigo. Cada enemigo tiene un identificador nico, una
     * posicin, una direccin inicial, un nombre y una distancia
     * @param posicionX Posicin inicial X del enemigo
     * @param posicionY Posicin inicial Y del enemigo
     * @param direccion Direccin inicial del enemigo
     * @param nombre    Nombre del enemigo
     * @param distancia Direccin inicial del enemigo
    */
    public Enemigo(int posicionX, int posicionY, int direccion, String nombre, int color) {
        super(posicionX, posicionY, direccion);
        contadorEnemigos++;
        this.idEnemigo = contadorEnemigos;
        this.nombre = nombre;
        this.color = color;
        this.distancia = 0;
    }

    /**
     * Devuelve la letra inicial del nombre del enemigo
     * @return la letra inicial del nombre del enemigo
     */
    public String getInicial() {
        return nombre.substring(0,1);
    }
    
    /**
     * Mtodo recursivo que encuentra el camino hasta una casilla indicada desde la posicin
     * inicial del enemigo.
     * @param x0       Posicin X inicial del enemigo
     * @param y0       Posicin Y inicial del enemigo
     * @param casillas Array de casillas del tablero
     * @param enemigos Lista de enemigos
     * @param x1       Posicin X de la casilla objetivo
     * @param y1       Posicin Y de la casilla objetivo
     * @param i        Contador de casillas recorridas (nmero de llamadas recursivas)
     * @return true si se ha encontrado un camino, false si no
     */
    public boolean resuelveCamino(int x0, int y0, Casilla[][] casillas, Lista enemigos, int x1, int y1, int i) {
    	// Se comprueba si la posicin inicial coincide con la objetivo
    	if(x0==x1 && y0==y1){
    		
    		// Se almacena como distancia el valor del contador y se devuelve true
    		distancia = i;
    		return true;
    	}
    	// Se incrementa el contador de casillas recorridas
    	i++;
    	
    	// Se marca la casilla como no pisable para que el enemigo no la considere como
    	// pisable en sucesivas llamadas, evitando posibles bucles infinitos.
    	casillas[y0][x0].setPisable(false);
    	
    	// Se construyen los cuatro posibles movimientos (se estima la distancia en el proceso)
    	Movimiento[] movimientos = new Movimiento[4];
   		movimientos[0] = new Movimiento(x0,y0-1,DIRECCION_ARRIBA,x1,y1);
   		movimientos[1] = new Movimiento(x0,y0+1,DIRECCION_ABAJO,x1,y1);      
  		movimientos[2] = new Movimiento(x0-1,y0,DIRECCION_IZQUIERDA,x1,y1);
  		movimientos[3] = new Movimiento(x0+1,y0,DIRECCION_DERECHA,x1,y1);  		
 
  		// Se crea la lista de movimientos
       	ListaMovimientos lista = new ListaMovimientos(movimientos);
       	
       	// Se ordena la lista de menor a mayor en funcin de la distancia estimada
       	lista.ordenar(lista.getLongitud()-1);
       	
       	// Se recorre la lista de movimientos empezando por los de menor distancia estimada 
        for(int j=0; j<lista.getLongitud(); j++) {
        	Movimiento m = lista.getMovimiento(j);
        	
        	// Se comprueba si la casilla destino es pisable
          	if (esPisable(m.getPosicionX(),m.getPosicionY(),casillas,enemigos,i)){

          		// Se llama recursivamente a resuelveCamino desde la casilla destino
          		boolean tmp = resuelveCamino(m.getPosicionX(),m.getPosicionY(),casillas,enemigos,x1,y1,i);
    			
          		// Se comprueba si se ha encontrado un camino
          		if (tmp){
          			
          			// Se almacena la direccin del primer movimiento
    				direccion = m.getDireccion();
    				
    				// Si la casilla no es una guarida se vuelve a marcar como pisable
    				if (!casillas[y0][x0].esGuarida())casillas[y0][x0].setPisable(true);
    				return true;
    			}
    		}
    	}
		// Si la casilla no es una guarida se vuelve a marcar como pisable
		if (!casillas[y0][x0].esGuarida())casillas[y0][x0].setPisable(true);
    	return false;
    } 	
    	
    	
    /**
     * Mtodo para indicar al enemigo que se mueva. Para ello se le indican las casillas
     * del tablero, el resto de enemigos y el protagonista
     * @param casillas Casillas del tablero
     * @param enemigos Todos los enemigos del juego
     * @param x        Posicin X de la casilla objetivo
     * @param y        Posicin Y de la casilla objetivo
     */
    public void muevete(Casilla[][] casillas, Lista enemigos, int x, int y) {
		// Se llama al mtodo recursivo para que encuentre un camino
    	boolean encontrado = resuelveCamino(posicionX,posicionY,casillas,enemigos,x,y,0);
    	
    	// Si lo encuentra se llama al mtodo de la superclase para completar el movimiento
    	if (encontrado) super.muevete(casillas);
    }
 
    /**
     * Mtodo privado que comprueba si una determinada casilla es pisable por el enemigo
     * Para ello comprueba que la casilla sea pisable. Adems, si se trata del movimiento
     * inicial (que es el que se completar) se comprueba que la casilla no est ocupada
     * por otro enemigo.
     * @param x Posicin x de la casilla a comprobar
     * @param y Posicin y de la casilla a comprobar
     * @param casillas Casillas del tablero
     * @param enemigos Todos los enemigos del tablero
     * @return true si es pisable, false si no lo es
     */
    private boolean esPisable(int x, int y, Casilla[][] casillas, Lista enemigos,int numMovimiento) {
    	// Se almacena si la casilla es pisable
    	boolean pisable = casillas[y][x].esPisable();
    	
    	// En el caso de que sea pisable, pero sea el primer movimiento del camino
        if (pisable && numMovimiento==1){
        	
        	// Se recorre la lista de enemigos
        	for(int i=0; i<enemigos.getLongitud(); i++) {
        		
        		// Si alguno de ellos ya ocupa la casilla, se marca como no pisable
        		if(enemigos.getEnemigo(i).coincide(x,y)) {
        			pisable = false;
        			break;
        		}
        	}
        }
        return pisable;
    }
 
    /**
     * Devuelve el identificador nico del enemigo
     * @return el identificador nico del enemigo
     */
    public int getIdEnemigo() {
        return idEnemigo;
    }

    /**
     * Devuelve el nombre del enemigo
     * @return el nombre del enemigo
     */
    public String getNombre() {
        return nombre;
    }

    /**
     * Devuelve el color del enemigo
     * @return el color del enemigo
     */
    public String getColor() {
        switch(color) {
            case COLOR_ROJO:
                return "roj";
            case COLOR_NARANJA:
                return "nar";
            case COLOR_AMARILLO:
                return "ama";
            case COLOR_VERDE:
            	return "ver";
            case COLOR_CELESTE:
            	return "cel";
            case COLOR_AZUL:
				return "azu";
            case COLOR_VIOLETA:
				return "vio";
            case COLOR_ROSA:
				return "ros";
            default:
                return "";
        }
    }
    
    /**
     * Devuelve la distancia del enemigo
     * @return distancia del enemigo
     */
    public int getDistancia() {
        return distancia;
    }

    /**
     * Mtodo que comprueba si un determinado nombre coincide con el del enemigo actual
     * @param nombre nombre 
     * @return true si se trata del mismo nombre y false en caso contrario
     */
    public boolean esEnemigo(String nombre) {
	if (this.nombre.equals(nombre)) return true;
	else return false;
    }

    /**
     * Mtodo privado que comprueba si el enemigo actual es menor que otro
     * @param e Enemigo
     * @return true si la distancia es menor que el enemigo e, false si no lo es
     */
    public boolean menor(Enemigo e) {
	if (this.distancia<e.getDistancia()) return true;
	else return false;
    }

    /**
     * Mtodo privado que comprueba si el enemigo actual es mayor que otro
     * @param e Enemigo
     * @return true si la distancia es mayor que el enemigo e, false si no lo es
     */
    public boolean mayor(Enemigo e) {
	if (this.distancia>e.getDistancia()) return true;
	else return false;
    }

    /**
     * Mtodo privado que comprueba si el enemigo actual es igual que otro
     * @param e Enemigo
     * @return true si la distancia es igual que el enemigo e, false si no lo es
     */
    public boolean igual(Enemigo e) {
	if (this.distancia==e.getDistancia()) return true;
	else return false;
    }
}
